/****************************************************
* Zeus CMD - C++ Tutorial 53 : Operator Overloading *
* By Grant James (ZEUS)                             *
* http://www.zeuscmd.com                            *
****************************************************/
#include <iostream>
#include <stdlib.h>

using namespace std;

class Money
{
public :

	Money() {}
	Money(int d, int c) :
		dollars(d),
		cents(c) {}

	Money operator+(const Money &rhs) const;
	Money &operator+=(const Money &rhs);
	Money &operator++();
	const Money operator++(int);

	void printValue();

private :
	// Getters and Setters
	//---------------------
	int getDollars() const
	{ return dollars; }
	void setDollars(int d)
	{ dollars = d; }
	int getCents() const
	{ return cents; }
	void setCents(int c)
	{ cents = c; }

private :

	int dollars;
	int cents;
};

void Money::printValue()
{
	cout << dollars;
	cout << ".";

	if (!cents)
	{
		cout << "00";
	}
	else
	{
		if (cents < 10)
			cout << "0";

		cout << cents;
	}

	cout << endl;
}

Money Money::operator+(const Money &rhs) const
{
	Money temp;

	temp.setDollars(this->dollars + rhs.getDollars());
	temp.setCents(this->cents + rhs.getCents());

	int incdollar = temp.getCents() / 100;
	temp.setCents(temp.getCents() % 100);
	temp.setDollars(temp.getDollars() + incdollar);

	return temp;
}

Money &Money::operator+=(const Money &rhs)
{
	*this = *this + rhs;

	return *this;
}

Money &Money::operator++()
{
	this->dollars += 1;

	return *this;
}

const Money Money::operator++(int)
{
	Money temp = *this;

	this->dollars += 1;

	return temp;
}

int main()
{
	Money a(50, 65);
	Money b(32, 42);
	Money c(0, 3);

	// Addition
	cout << "Addition" << endl;
	(a + b).printValue();
	(a + b + c).printValue();

	Money d = c;
	d.printValue();

	// Increase
	cout << "Increment" << endl;
	(c += c).printValue();
	(c += c + a).printValue();
	c.printValue();
	a.printValue();

	// Postfix & Prefix
	cout << "Postfix & Prefix" << endl;
	(++c + d).printValue();
	(d++ + a).printValue();
	d.printValue();

	++++++c;
	//c++++++; // Error

	system("sleep 1");

	return 0;
}
